using System;
using System.Collections.Generic;
using System.Configuration;
using SimpleJSON;
using MacGruber;
using UnityEngine;

namespace CheesyFX
{
    public class FlexIt : MVRScript
    {
        private static string morphPath = "Custom/Atom/Person/Morphs/female/CheesyFX/FlexIt/";
        private List<FlexRegion> flexRegions = new List<FlexRegion>();
        private static float _speed = 10f;
        public static float speed  = _speed * Time.fixedDeltaTime;
        
        private static GenerateDAZMorphsControlUI morphUI;
        private UIDynamicTabBar regionTabbar;

        private string[] regionNames = { "Knee", "Thigh", "Abdomen"};
        private string[][] rbNames = { new[]{"Shin"}, new[]{ "Thigh" }, new[]{ "pelvis", "chest" }};

        public void OnEnable()
        {
            for (int i = 0; i < flexRegions.Count; i++)
            {
                flexRegions[i].enabled = true;
                // flexRegions[i].OnDisable();
            }
        }

        public override void Init()
        {
            DAZCharacterSelector dcs = containingAtom.GetComponentInChildren<DAZCharacterSelector>();
            if (dcs.selectedCharacter.isMale)
            {
                morphPath = morphPath.Replace("female", "male");
            }
            if (Utils.IsInPackage(this))
            {
                var packageUid = Utils.GetPluginPath(this).Split(':')[0] + ":/";
                // factoryDefaults = packageUid + factoryDefaults;
                morphPath = packageUid + morphPath;
            }
            morphUI = dcs.morphsControlUI;
            
            for(int i=0; i<regionNames.Length; i++)
            {
                var flexRegion = gameObject.AddComponent<FlexRegion>();
                flexRegion.Init(i, regionNames[i], rbNames[i], i<2);
                flexRegions.Add(flexRegion);
            }
            
            UIManager.OnInitUI(CreateUIElement, leftUIElements, rightUIElements);
            CreateUI();
        }
        
        private UIDynamicV4Slider sRatioUid;
        private UIDynamicV4Slider aStrengthUid;
        private UIDynamicV4Slider gStrengthUid;
        private UIDynamicV4Slider minAccelUid;
        private UIDynamicV4Slider maxMorphUid;
        private void CreateUI()
        {
            regionTabbar = UIManager.CreateTabBar(regionNames, val => RegisterVectors(), script:this, invokeDefault:false);

            maxMorphUid = UIManager.CreateV4Slider(script: this);
            sRatioUid = UIManager.CreateV4Slider(script: this, constrained:true);
            aStrengthUid = UIManager.CreateV4Slider(script: this);

            // for (int i = 0; i < 4; i++)
            // {
            //     sRatioUid.sliders[i].sliderControl.clamp = true;
            //     sRatioUid.sliders[i].rangeAdjustEnabled = false;
            // }
            
            System.Text.StringBuilder sb = new System.Text.StringBuilder();
            sb.AppendLine("<b>Amount:</b>");
            sb.AppendLine("The maximum morph value achievable.");
            sb.AppendLine("");
            sb.AppendLine("<b>S-Ratio:</b>");
            sb.AppendLine("Sets which portion of the max morph value can be achieved in static poses. If <1, full morphing is only achievable through dynamic effects (angular acceleration).");
            sb.AppendLine("0 means only dynamic effects and 1 leads to pose based morphing only.");
            sb.AppendLine("");
            sb.AppendLine("<b>Dynamic</b>");
            sb.AppendLine("The strength of the dynamic effects.");
            Utils.SetupInfoText(this, sb.ToString(), 500f, false);
            // gStrengthUid = UIManager.CreateV4Slider(script: this);
            // minAccelUid = UIManager.CreateV4Slider(script: this);
            
            maxMorphUid.setToggleCallbackFunction += maxMorphUid.ToggleSync;
            maxMorphUid.toggle.label = "Sync";
            sRatioUid.setToggleCallbackFunction += sRatioUid.ToggleSync;
            sRatioUid.toggle.label = "Sync";
            aStrengthUid.setToggleCallbackFunction += aStrengthUid.ToggleSync;
            aStrengthUid.toggle.label = "Sync";
            
            regionTabbar.SelectTab(0);
        }

        private void RegisterVectors()
        {
            var flexRegion = flexRegions[regionTabbar.id];
            sRatioUid.RegisterVector(flexRegion.sRatioJ, labelsuffixes:new []{"fwd", "bwd", "in", "out"});
            aStrengthUid.RegisterVector(flexRegion.aStrengthJ, labelsuffixes:new []{"fwd", "bwd", "in", "out"});
            // gStrengthUid.RegisterVector(flexRegion.gStrengthJ, labelsuffixes:new []{"fwd", "bwd", "in", "out"});
            // minAccelUid.RegisterVector(flexRegion.minAccelJ, labelsuffixes:new []{"fwd", "bwd", "in", "out"});
            maxMorphUid.RegisterVector(flexRegion.maxMorphJ, labelsuffixes:new []{"fwd", "bwd", "in", "out"});
            

            for (int i = 0; i < 4; i++)
            {
                bool active = i < flexRegion.morphsCount;
                sRatioUid.SliderSetActive(i, active);
                aStrengthUid.SliderSetActive(i, active);
                maxMorphUid.SliderSetActive(i, active);
            }
            // flexRegion.sRatioJ.constrained = true;
            // sRatioUid.SetConstrained(true);
            
        }

        public void FixedUpdate()
        {
            speed = _speed * Time.fixedDeltaTime;
        }

        public void OnDisable()
        {
            for (int i = 0; i < flexRegions.Count; i++)
            {
                flexRegions[i].enabled = false;
            }
        }
        
        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            for (int i = 0; i < flexRegions.Count; i++)
            {
                flexRegions[i].Reset();
            }
            JSONClass jc = base.GetJSON(includePhysical, includeAppearance, needsStore);
            return jc;
        }

        public static DAZMorph GetMorphByUid(string name)
        {
            DAZMorph morph;
            string uid = morphPath + name +".vmi";
            morph = morphUI.GetMorphByUid(uid);
            // if(morph == null) $"Morph '{uid}' not found!".Print();
            return morph;
        }
    }
}